package didatico.armazenamento.bancodados.listaexercicios.lista03;

import java.io.IOException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import didatico.armazenamento.bancodados.lib.Database;

public class L03Q02
{
    public static void main(String argumentos[])
    {
        try {
            Database banco = Database.getInstance();
            Connection conexao = banco.getConnection();
            
            // atualizaContas(conexao);
            atualizaContasSemNegativos(conexao);
            
            banco.releaseConnection();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (SQLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    public static void atualizaContas(Connection conexao)
           throws SQLException
    {
        Statement comandoSelect = conexao.createStatement();
        
        PreparedStatement comandoUpdate = conexao.prepareStatement(
            "UPDATE ContaCorrente " +
                   "SET Saldo = Saldo + ? " +
                   "WHERE NumeroConta = ?");
        
        ResultSet resultado = comandoSelect.executeQuery(
            "SELECT NumeroConta, ValorMovimentacao, DebitoCredito " +
                   "FROM Movimento");
        
        while (resultado.next())
        {
            String conta = resultado.getString("NumeroConta");
            float valor = resultado.getFloat("ValorMovimentacao");
            String dc = resultado.getString("DebitoCredito");
            
            valor = (dc.equalsIgnoreCase("C")) ? valor : -valor;
            
            System.out.println("Atualizando: " + conta + "; " +
                               valor);
            
            comandoUpdate.setFloat(1, valor);
            comandoUpdate.setString(2, conta);
            comandoUpdate.executeUpdate();
        }
        
        comandoSelect.close();
        comandoUpdate.close();
    }

    public static void atualizaContasSemNegativos(Connection conexao)
           throws SQLException
    {
        Statement comandoCreate = conexao.createStatement();
        comandoCreate.executeUpdate(
            "CREATE TABLE MovimentosInvalidos (" +
                "NumeroConta VARCHAR(6) NOT NULL, " +
                "DataMovimentacao DATE, " +
                "ValorMovimentacao FLOAT, " +
                "DebitoCredito VARCHAR(1), " +
                "CodigoMovimento VARCHAR(20), " +
                "PRIMARY KEY (NumeroConta, DataMovimentacao))");
        
        Statement comandoSelect1 = conexao.createStatement();
         
        PreparedStatement comandoUpdate = conexao.prepareStatement(
            "UPDATE ContaCorrente " +
                   "SET Saldo = Saldo + ? " +
                   "WHERE NumeroConta = ?");
        PreparedStatement comandoInsert = conexao.prepareStatement(
            "INSERT INTO MovimentosInvalidos " +
                   "VALUES (?, ?, ?, ?, ?)");
        PreparedStatement comandoSelect2 = conexao.prepareStatement(
            "SELECT Saldo FROM ContaCorrente WHERE NumeroConta = ?");
         
        ResultSet resultado = comandoSelect1.executeQuery(
            "SELECT * FROM Movimento");
         
        while (resultado.next())
        {
            String conta = resultado.getString("NumeroConta");
            Date datamov = resultado.getDate("DataMovimentacao");
            float valor = resultado.getFloat("ValorMovimentacao");
            String dc = resultado.getString("DebitoCredito");
            String codigo = resultado.getString("CodigoMovimento");
            
            valor = (dc.equalsIgnoreCase("C")) ? valor : -valor;
             
            System.out.println("Atualizando: " + conta + "; " +
                               valor);
            
            comandoSelect2.setString(1, conta);
            ResultSet rssaldo = comandoSelect2.executeQuery();
            rssaldo.next();
            float saldoAtual = rssaldo.getFloat("Saldo");
            
            if (saldoAtual + valor >= 0) {
                comandoUpdate.setFloat(1, valor);
                comandoUpdate.setString(2, conta);
                comandoUpdate.executeUpdate();
            } else {
                comandoInsert.setString(1, conta);
                comandoInsert.setDate(2, datamov);
                comandoInsert.setFloat(3, -valor);
                comandoInsert.setString(4, dc);
                comandoInsert.setString(5, codigo);
                comandoInsert.executeUpdate();
            }
        }
         
        comandoSelect1.close();
        comandoUpdate.close();
        comandoInsert.close();
        comandoSelect2.close();
    }

    /**
     * Este metodo utiliza a seguinte Stored Procedure:
     *     CREATE PROCEDURE atualizaSaldo(conta VARCHAR(20), valor FLOAT)
     *        UPDATE ContaCorrente
     *               SET Saldo = Saldo + valor
     *               WHERE NumeroConta = conta;
     * @param conexao
     * @throws SQLException
     */
    public static void atualizaContasSemNegativosSP(Connection conexao)
                       throws SQLException
    {
        conexao.setAutoCommit(false);

        Statement comandoSelect1 = conexao.createStatement();
        ResultSet resultado = comandoSelect1.executeQuery("SELECT * FROM Movimento");

        PreparedStatement comandoSelect2 = conexao.prepareStatement(
                "SELECT Saldo FROM ContaCorrente WHERE NumeroConta = ?");
        CallableStatement comandoProcedure = conexao.prepareCall(
                "{ CALL atualizaSaldo(?, ?) }");
        PreparedStatement comandoInsert = conexao.prepareStatement(
                "INSERT INTO MovimentoInvalido VALUES (?, ?, ?, ?, ?)");
        
        while (resultado.next()) {
            String conta = resultado.getString("NumeroConta");
            float valor = resultado.getFloat("ValorMovimentacao");
            String dc = resultado.getString("DebitoCredito");
        
            comandoSelect2.setString(1, conta);
            ResultSet resultado2 = comandoSelect2.executeQuery();
            resultado2.next();
            float saldo = resultado2.getFloat("Saldo");
        
            comandoProcedure.setString(1, conta);
            if (dc.equals("C")) {
                comandoProcedure.setFloat(2, valor);
                comandoProcedure.execute();
            } else if (dc.equals("D") && valor <= saldo) {
                comandoProcedure.setFloat(2, -valor);
                comandoProcedure.execute();
            } else if (dc.equals("D") && valor > saldo) {
                comandoInsert.setString(1, resultado.getString(1));
                comandoInsert.setDate(2, resultado.getDate(2));
                comandoInsert.setFloat(3, resultado.getFloat(3));
                comandoInsert.setString(4, resultado.getString(4));
                comandoInsert.setString(5, resultado.getString(5));
                comandoInsert.executeUpdate();
            }
        }
        conexao.commit();
        conexao.setAutoCommit(true);
  }
}
