package didatico.concorrente.pack04sincronizacao;

/**
 * Exemplo com threads paralelas que funcionam corretamente;
 * transferencia bem sucedida do synchronized para clientes
 * (MultiplasContagens) -- solucao: synchronized aponta para
 * o mesmo objeto.
 * 
 * @author Andre Santanche
 *
 */
public class Ex05ContaCerta extends Thread
{
    public class Contador
    {
        private int valor = 0;
        
        public int getValor()
        {
            return valor;
        }
        
        public void atualiza(int quantidade)
        {
            int x = valor;

            try {
                Thread.sleep((long)(Math.random() * 2));
            } catch (InterruptedException e) {
                /* nada */
            }
            
            valor = x + quantidade;

            try {
                Thread.sleep((long)(Math.random() * 2));
            } catch (InterruptedException e) {
                // nada
            }
        }
    }
    
    public class MultiplasContagens extends Thread
    {
        private Contador umContador;
        private int quantidade;
        
        public MultiplasContagens(Contador umContador, int quantidade)
        {
            this.umContador = umContador;
            this.quantidade = quantidade;
        }
        
        public void run()
        {
            for (int i = 0; i < 10; i++)
                synchronized (umContador) {
                    umContador.atualiza(quantidade);
                }
        }
    }
    
    public void executaContagem()
    {
        Contador umContador = new Contador();
        
        System.out.print("antes: " + umContador.getValor() + "; ");
        
        MultiplasContagens mult1 = new MultiplasContagens(umContador, 1),
                           mult2 = new MultiplasContagens(umContador, -1);
        
        mult1.start();
        mult2.start();
        
        try {
            mult1.join();
            mult2.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        System.out.println("depois: " + umContador.getValor());
    }
    
    public static void main(String args[])
    {
        Ex05ContaCerta contaCerta = new Ex05ContaCerta();
        
        for (int j = 0; j < 10; j++)
            contaCerta.executaContagem();
    }
}
